#!/usr/bin/python
# -*- coding: UTF-8 -*-

   #########################################################################
 ##                                                                       ##
##              ┏┓╻┏━┓╻ ╻╺┳╸╻╻  ╻ ╻┏━┓   ╺┳╸┏━╸┏━┓┏┳┓╻┏┓╻┏━┓╻             ##
##              ┃┗┫┣━┫┃ ┃ ┃ ┃┃  ┃ ┃┗━┓    ┃ ┣╸ ┣┳┛┃┃┃┃┃┗┫┣━┫┃             ##
##              ╹ ╹╹ ╹┗━┛ ╹ ╹┗━╸┗━┛┗━┛    ╹ ┗━╸╹┗╸╹ ╹╹╹ ╹╹ ╹┗━╸           ##
##                  — An integrated terminal for nautilus —               ##
##                                                                        ##
############################################################################
##                                                                        ##
## Nautilus Terminal - An integrated terminal for nautilus                ##
##                                                                        ##
## Copyright (C) 2010  Fabien Loison (flo@flogisoft.com)                  ##
##                                                                        ##
## This program is free software: you can redistribute it and/or modify   ##
## it under the terms of the GNU General Public License as published by   ##
## the Free Software Foundation, either version 3 of the License, or      ##
## (at your option) any later version.                                    ##
##                                                                        ##
## This program is distributed in the hope that it will be useful,        ##
## but WITHOUT ANY WARRANTY; without even the implied warranty of         ##
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          ##
## GNU General Public License for more details.                           ##
##                                                                        ##
## You should have received a copy of the GNU General Public License      ##
## along with this program.  If not, see <http://www.gnu.org/licenses/>.  ##
##                                                                        ##
############################################################################
##                                                                        ##
## WEB SITE : http://software.flogisoft.com/nautilus-terminal/            ##
##                                                                       ##
#########################################################################


"""An integrated terminal for nautilus"""

__author__ = "Fabien LOISON <flo@flogisoft.com>"
__version__ = "0.2"
__appname__ = "nautilus-terminal"


import os
import re
import urllib
import gettext
gettext.install(__appname__)

import gtk
import pygtk
pygtk.require("2.0")
import pango
import vte

import nautilus


TERMINAL_GUI_FILE = "/usr/share/nautilus-terminal/nautilus-terminal.glade"
PREF_GUI_FILE = "/usr/share/nautilus-terminal/nautilus-terminal-preferences.glade"
CONF_FILE = os.path.join(os.environ.get("HOME"), ".nautilus-terminal.conf")

CURSOR_SHAPE = [
        _("Block"),
        _("Vertical bar"),
        _("Underlined")
        ]

PREDEF_PALETTE = {
        'Tango': [
            "#000000", "#cc0000", "#4e9a06", "#c4a000",
            "#3465a4", "#75507b", "#06989a", "#d3d7cf",
            "#555753", "#ef2929", "#8ae234", "#fce94f",
            "#729fcf", "#ad7fa8", "#34e2e2", "#ffffff"
            ],
        'Linux console': [
            "#000000", "#aa0000", "#00aa00", "#aa5500",
            "#0000aa", "#aa00aa", "#00aaaa", "#aaaaaa",
            "#555555", "#ff5555", "#55ff55", "#ffff55",
            "#5555ff", "#ff55ff", "#55ffff", "#ffffff"
            ],
        'XTerm': [
            "#000000", "#cd0000", "#00cd00", "#cdcd00",
            "#1e90ff", "#cd00cd", "#00cdcd", "#e5e5e5",
            "#4c4c4c", "#ff0000", "#00ff00", "#ffff00",
            "#4682b4", "#ff00ff", "#00ffff", "#ffffff"
            ],
        'Rxvt': [
            "#000000", "#cd0000", "#00cd00", "#cdcd00",
            "#0000cd", "#cd00cd", "#00cdcd", "#faebd7",
            "#404040", "#ff0000", "#00ff00", "#ffff00",
            "#0000ff", "#ff00ff", "#00ffff", "#ffffff"
            ],
        'Evening': [
            "#333333", "#c75959", "#57af57", "#f5ce54",
            "#4b7bba", "#ca68ca", "#8bcfcf", "#a1aeaf",
            "#757575", "#ffa0a0", "#90ee90", "#ffff60",
            "#add8e6", "#ffbbff", "#e0ffff", "#ffffff"
            ],
        'Fruity': [
            "#111111", "#d1070d", "#4e9a06", "#fb660a",
            "#1066a9", "#ea037c", "#16afc6", "#8598a3",
            "#373737", "#d1070d", "#4e9a06", "#fb660a",
            "#1066a9", "#ea037c", "#16afc6", "#ffffff"
            ]
        }


class Conf(dict):
    """Read/Write configuration"""

    def __init__(self, conf_file=None):
        """The constructor"""
        self._conf_file = conf_file
        #Initialize the dictionary
        dict.__init__(self)
        #General
        self['general_defheight'] = 7
        self['general_starthidden'] = False
        self['general_command'] = "/bin/bash"
        self['general_cursor'] = 0
        #Color
        self['color_text'] = "#ffffff"
        self['color_background'] = "#000000"
        self['color_palettename'] = "Tango"
        self['color_palette'] = []
        #Font
        self['font_name'] = "monospace 10"
        self['font_allowbold'] = True
        #Folders
        self['folders_showinall'] = True
        self['folders_list'] = []
        #Read configuration
        if self._conf_file != None:
            self.import_user_conf()
            self._check_conf()

    def import_user_conf(self):
        """Import user configuration file."""
        if os.path.isfile(self._conf_file):
            current_section = "unknown"
            user_conf_file = open(self._conf_file, "r")
            for line in user_conf_file:
                #Comments
                if re.match(r"\s*#.*", line):
                    continue
                #Section
                elif re.match(r"\s*\[([a-z]+)\]\s*", line.lower()):
                    match = re.match(r'\s*\[([a-z]+)\]\s*', line.lower())
                    current_section = match.group(1)
                #Boolean key
                elif re.match(r"\s*([a-z]+)\s*=\s*(yes|no|true|false)\s*", line.lower()):
                    match = re.match(r"\s*([a-z]+)\s*=\s*(yes|no|true|false)\s*", line.lower())
                    key = match.group(1)
                    value = match.group(2)
                    if value in ("yes", "true"):
                        value = True
                    else:
                        value = False
                    self[current_section + "_" + key] = value
                #String key : path
                elif re.match(r"\s*(path|PATH|Path)\s*=\s*\"(.+)\"\s*", line):
                    match = re.match(r"\s*(path|PATH|Path)\s*=\s*\"(.+)\"\s*", line)
                    key = "list"
                    value = match.group(2)
                    self[current_section + "_" + key].append(value)
                #String key : palette
                elif re.match(r"\s*(palette)\s*=\s*\"([0-9a-f#;]+)\"\s*", line.lower()):
                    match = re.match(r"\s*(palette)\s*=\s*\"([0-9a-f#;]+)\"\s*", line.lower())
                    key = "palette"
                    value = match.group(2)
                    self[current_section + "_" + key] = value.split(";")
                #String key
                elif re.match(r"\s*([a-z]+)\s*=\s*\"(.+)\"\s*", line):
                    match = re.match(r"\s*([a-z]+)\s*=\s*\"(.+)\"\s*", line)
                    key = match.group(1)
                    value = match.group(2)
                    self[current_section + "_" + key] = value
                #Integer key
                elif re.match(r"\s*([a-z]+)\s*=\s*([0-9]+)\s*", line.lower()):
                    match = re.match(r"\s*([a-z]+)\s*=\s*([0-9]+)\s*", line.lower())
                    key = match.group(1)
                    value = match.group(2)
                    self[current_section + "_" + key] = int(value)
            user_conf_file.close()

    def save_user_conf(self):
        """Save configuration file."""
        try:
            user_conf_file = open(self._conf_file, "w")
            #Warning
            user_conf_file.write("#" + _("Configuration written by Nautilus Terminal") + "\n")
            user_conf_file.write("#" + _("Please edit with caution") + "\n")
            #General
            user_conf_file.write("\n[GENERAL]\n")
            user_conf_file.write(self._write_int("general_defheight"))
            user_conf_file.write(self._write_bool("general_starthidden"))
            user_conf_file.write(self._write_str("general_command"))
            user_conf_file.write(self._write_int("general_cursor"))
            #Color
            user_conf_file.write("\n[COLOR]\n")
            user_conf_file.write(self._write_str("color_text"))
            user_conf_file.write(self._write_str("color_background"))
            user_conf_file.write(self._write_str("color_palettename"))
            user_conf_file.write(self._write_palette("color_palette"))
            #Font
            user_conf_file.write("\n[FONT]\n")
            user_conf_file.write(self._write_str("font_name"))
            user_conf_file.write(self._write_bool("font_allowbold"))
            #Folders
            user_conf_file.write("\n[FOLDERS]\n")
            user_conf_file.write(self._write_bool("folders_showinall"))
            user_conf_file.write(self._write_list("folders_list"))
            #End
            user_conf_file.write("\n\n") #I like blank lines at the end :)
        except OSError: #Permission denied
            print("E: [%s:Conf.save_user_conf] Can't write configuration (permission denied)" % __file__)
            return
        except IOError: #Input/Output error
            print("E: [%s:Conf.save_user_conf] Can't write configuration (IO error)" % __file__)
            return
        else:
            user_conf_file.close()

    def _check_conf(self):
        """Check configuration"""
        #General
        if not os.path.isfile(str(self['general_command'])):
            self['general_command'] = "/bin/bash"
        if int(self['general_defheight']) < 4:
            self['general_defheight'] = 4
        elif int(self['general_defheight']) > 20:
            self['general_defheight'] = 20
        if not int(self['general_cursor']) in [0, 1, 2]:
            self['general_cursor'] = 0
        #Color
        if not self['color_palettename'] in PREDEF_PALETTE and self['color_palettename'] != "Custom":
            self['color_palettename'] = "Tango"
        if len(self['color_palette']) != 16:
            self['color_palette'] = list(PREDEF_PALETTE['Tango'])
        #Folders
        if len(self['folders_list']) == 0:
            self['folders_showinall'] = True

    def _write_bool(self, key):
        """Return the string to write in config file for boolean key
        
        Argument:
          * key -- the name of the CONF key
        """
        wkey = key.split("_")[1]
        if self[key]:
            value = "Yes"
        else:
            value = "No"
        return "\t%s = %s\n" % (wkey, value)

    def _write_list(self, key):
        """Return the string to write in config file for list key (path)
        
        Argument:
          * key -- the name of the CONF key
        """
        result = ""
        for value in self[key]:
            result += "\tpath = \"%s\"\n" % value
        return result

    def _write_palette(self, key):
        """Return the string to write in config file for palette
        
        Argument:
          * key -- the name of the CONF key
        """
        wkey = key.split("_")[1]
        value = ""
        for color in self[key]:
            value += ";%s" % color
        return "\t%s = \"%s\"\n" % (wkey, value[1:])

    def _write_str(self, key):
        """Return the string to write in config file for string key
        
        Argument:
          * key -- the name of the CONF key
        """
        wkey = key.split("_")[1]
        value = self[key]
        return "\t%s = \"%s\"\n" % (wkey, value)

    def _write_int(self, key):
        """Return the string to write in config file for integer key
        
        Argument:
          * key -- the name of the CONF key
        """
        wkey = key.split("_")[1]
        value = self[key]
        return "\t%s = %i\n" % (wkey, value)


class NautilusTerminalPref(object):
    """Preferences window for Nautilus Terminal."""

    def __init__(self, callback, terminal):
        """The constructor."""
        self._callback = callback
        self._terminal = terminal
        self._conf = Conf(CONF_FILE)
        #Change current path to home dir
        os.chdir(os.environ.get("HOME"))
        #GUI
        self.gui = gtk.Builder()
        self.gui.set_translation_domain(__appname__)
        self.gui.add_from_file(PREF_GUI_FILE)
        self.gui.connect_signals(self)
        #### winMain ####
        self.winMain = self.gui.get_object("winMain")
        #demoTerm
        self.demoTerm = vte.Terminal()
        self.demoTerm.set_size(20, 2)
        self.demoTerm.fork_command("/usr/share/nautilus-terminal/preview.sh")
        self.demoTerm.show()
        #General
        self.cbStartHidden = self.gui.get_object("cbStartHidden")
        self.spbtnDefHeight = self.gui.get_object("spbtnDefHeight")
        self.entryCmd = self.gui.get_object("entryCmd")
        self.comboboxCursor = self.gui.get_object("comboboxCursor")
        self.lsstCursor = gtk.ListStore(str)
        self.comboboxCursor.set_model(self.lsstCursor)
        self.cellCursor = gtk.CellRendererText()
        self.comboboxCursor.pack_start(self.cellCursor, True)
        self.comboboxCursor.add_attribute(self.cellCursor, "text", 0)
        for shape in CURSOR_SHAPE:
            self.lsstCursor.append([shape])
        #Color
        self.comboboxPalette = self.gui.get_object("comboboxPalette")
        self.lsstPalette = gtk.ListStore(str)
        self.comboboxPalette.set_model(self.lsstPalette)
        self.cellPallette = gtk.CellRendererText()
        self.comboboxPalette.pack_start(self.cellPallette, True)
        self.comboboxPalette.add_attribute(self.cellPallette, "text", 0)
        self._palette_list = []
        for palette in PREDEF_PALETTE:
            self._palette_list.append(palette)
            self.lsstPalette.append([palette])
        self._palette_list.append("Custom")
        gettext.install(__appname__) #Why it does not works whereas it was defined at the beginning ?
        self.lsstPalette.append([ _("Custom") ])
        self.clbtnFg = self.gui.get_object("clbtnFg")
        self.clbtnBg = self.gui.get_object("clbtnBg")
        self.clbtnPalette = []
        for i in range(0, 16):
            self.clbtnPalette.append(self.gui.get_object("clbtnPalette%i" % i))
        #Font
        self.fontbtn = self.gui.get_object("fontbtn")
        self.cbAllowBold = self.gui.get_object("cbAllowBold")
        #Folders
        self.rbAllFolders = self.gui.get_object("rbAllFolders")
        self.rbFolderList = self.gui.get_object("rbFolderList")
        self.btnFolderRemove = self.gui.get_object("btnRemoveFolder")
        self.trvFolderList = self.gui.get_object("trvFolderList")
        self.lsstFolderList = gtk.ListStore(str)
        self.trvFolderList.set_model(self.lsstFolderList)
        self.columnFolderList = gtk.TreeViewColumn("Path", gtk.CellRendererText(), text=0)
        self.trvFolderList.append_column(self.columnFolderList)
        #vboxTerm
        self.vboxTerm = self.gui.get_object("vboxTerm")
        self.vboxTerm.pack_start(self.demoTerm)
        #### winAbout ####
        self.winAbout = self.gui.get_object("winAbout")
        self.winAbout.connect("response", self.on_winAbout_response)
        self.winAbout.set_version(__version__)
        #### filechooser ####
        self.filechooser = self.gui.get_object("filechooser")
        ##
        self._put_opt()
        print self._conf['folders_list']

    #~~~~ winMAin ~~~~

    def on_btnCancel_clicked(self, widget, data=None):
        self.winMain.destroy()

    def on_btnOk_clicked(self, widget, data=None):
        global CONF
        self._conf._check_conf()
        CONF['folders_list'] = []
        for key in self._conf:
            if key != "folders_list":
                CONF[key] = self._conf[key]
            else:
                for path in self._conf['folders_list']:
                    CONF['folders_list'].append(path)
        CONF.save_user_conf()
        self._callback(self._terminal)
        self.winMain.destroy()

    def on_btnAbout_clicked(self, widget, data=None):
        self.winAbout.show()

    #~~~~ winMAin:General ~~~~

    def on_cbStartHidden_toggled(self, widget, data=None):
        self._conf['general_starthidden'] = widget.get_active()

    def on_spbtnDefHeight_value_changed(self, widget, data=None):
        self._conf['general_defheight'] = widget.get_value()

    def on_entryCmd_changed(self, widget, data=None):
        self._conf['general_command'] = widget.get_text()

    def on_comboboxCursor_changed(self, widget, data=None):
        self._conf['general_cursor'] = widget.get_active()
        self.demoTerm.set_cursor_shape(widget.get_active())

    #~~~~ winLMain:Color ~~~~

    def on_clbtnFg_color_set(self, widget, data=None):
        self._conf['color_text'] = str(widget.get_color())
        self._set_palette(self._conf['color_palettename'])

    def on_clbtnBg_color_set(self, widget, data=None):
        self._conf['color_background'] = str(widget.get_color())
        self._set_palette(self._conf['color_palettename'])

    def on_comboboxPalette_changed(self, widget, data=None):
        index = widget.get_active()
        self._conf['color_palettename'] = self._palette_list[index]
        self._set_palette(self._conf['color_palettename'])

    def on_clbtnPalette_color_set(self, widget, index=0):
        """General method for on_color_set"""
        changed_color = str(widget.get_color())
        if self._conf['color_palettename'] != "Custom":
            self._conf['color_palette'] = list(PREDEF_PALETTE[self._conf['color_palettename']])
            self._conf['color_palettename'] = "Custom"
            self.comboboxPalette.set_active(self._palette_list.index("Custom"))
        self._conf['color_palette'][index] = changed_color
        self._set_palette(self._conf['color_palettename'])

    def on_clbtnPalette0_color_set(self, widget, data=None):
        self.on_clbtnPalette_color_set(widget, 0)

    def on_clbtnPalette1_color_set(self, widget, data=None):
        self.on_clbtnPalette_color_set(widget, 1)

    def on_clbtnPalette2_color_set(self, widget, data=None):
        self.on_clbtnPalette_color_set(widget, 2)

    def on_clbtnPalette3_color_set(self, widget, data=None):
        self.on_clbtnPalette_color_set(widget, 3)

    def on_clbtnPalette4_color_set(self, widget, data=None):
        self.on_clbtnPalette_color_set(widget, 4)

    def on_clbtnPalette5_color_set(self, widget, data=None):
        self.on_clbtnPalette_color_set(widget, 5)

    def on_clbtnPalette6_color_set(self, widget, data=None):
        self.on_clbtnPalette_color_set(widget, 6)

    def on_clbtnPalette7_color_set(self, widget, data=None):
        self.on_clbtnPalette_color_set(widget, 7)

    def on_clbtnPalette8_color_set(self, widget, data=None):
        self.on_clbtnPalette_color_set(widget, 8)

    def on_clbtnPalette9_color_set(self, widget, data=None):
        self.on_clbtnPalette_color_set(widget, 9)

    def on_clbtnPalette10_color_set(self, widget, data=None):
        self.on_clbtnPalette_color_set(widget, 10)

    def on_clbtnPalette11_color_set(self, widget, data=None):
        self.on_clbtnPalette_color_set(widget, 11)

    def on_clbtnPalette12_color_set(self, widget, data=None):
        self.on_clbtnPalette_color_set(widget, 12)

    def on_clbtnPalette13_color_set(self, widget, data=None):
        self.on_clbtnPalette_color_set(widget, 13)

    def on_clbtnPalette14_color_set(self, widget, data=None):
        self.on_clbtnPalette_color_set(widget, 14)

    def on_clbtnPalette15_color_set(self, widget, data=None):
        self.on_clbtnPalette_color_set(widget, 15)

    #~~~~ winMain:Font ~~~~

    def on_fontbtn_font_set(self, widget, data=None):
        self._conf['font_name'] = str(self.fontbtn.get_font_name())
        font = pango.FontDescription(self.fontbtn.get_font_name())
        self.demoTerm.set_font(font)

    def on_cbAllowBold_toggled(self, widget, data=None):
        self._conf['font_allowbold'] = widget.get_active()
        self.demoTerm.set_allow_bold(widget.get_active())

    #~~~~ winMain:Folders ~~~~

    def on_rbAllFolders_toggled(self, widget, data=None):
        self._conf['folders_showinall'] = widget.get_active()

    def on_trvFolderList_cursor_changed(self, widget, data=None):
        model, iter_ = widget.get_selection().get_selected()
        if iter_ != None:
            self.btnFolderRemove.set_sensitive(True)

    def on_btnAddFolder_clicked(self, widget, data=None):
        self.filechooser.show()

    def on_btnRemoveFolder_clicked(self, widget, data=None):
        self._remove_path_from_list(
                self.trvFolderList,
                self.lsstFolderList,
                self._conf['folders_list']
                )
        self.btnFolderRemove.set_sensitive(False)

    #~~~~ winAbout ~~~~

    def on_winAbout_delete_event(self, widget, event):
        self.winAbout.hide()
        return True #Don't delete

    def on_winAbout_response(self, widget, response):
        if response < 0:
            self.winAbout.hide()

    #~~~~ filechooser ~~~~

    def on_filechooser_delete_event(self, widget, response):
        self.filechooser.hide()
        return True #Don't delete

    def on_btnFcOpen_clicked(self, widget, data=None):
        self._add_path_to_list(
                self.lsstFolderList,
                self.filechooser.get_filename(),
                self._conf['folders_list']
                )
        self.filechooser.hide()

    def on_btnFcCancel_clicked(self, widget, data=None):
        self.filechooser.hide()

    #~~~~

    def _put_opt(self):
        """Put options on the GUI"""
        #General
        self.cbStartHidden.set_active(self._conf['general_starthidden'])
        self.spbtnDefHeight.set_value(self._conf['general_defheight'])
        self.entryCmd.set_text(self._conf['general_command'])
        self.comboboxCursor.set_active(self._conf['general_cursor'])
        #Color
        self.clbtnFg.set_color(gtk.gdk.Color(self._conf['color_text']))
        self.clbtnBg.set_color(gtk.gdk.Color(self._conf['color_background']))
        index = self._palette_list.index(self._conf['color_palettename'])
        self.comboboxPalette.set_active(index)
        self._set_palette(self._conf['color_palettename'])
        #Font
        self.fontbtn.set_font_name(self._conf['font_name'])
        font = pango.FontDescription(self._conf['font_name'])
        self.demoTerm.set_font(font)
        self.cbAllowBold.set_active(self._conf['font_allowbold'])
        self.demoTerm.set_allow_bold(self._conf['font_allowbold'])
        #Folders
        if self._conf['folders_showinall']:
            self.rbAllFolders.set_active(True)
        else:
            self.rbFolderList.set_active(True)
        self._conf['folders_list'] = []
        for path in CONF['folders_list']:
            self._add_path_to_list(
                    self.lsstFolderList,
                    path,
                    self._conf['folders_list']
                    )
            
    def _set_palette(self, palette_name):
        if palette_name in PREDEF_PALETTE:
            colors = PREDEF_PALETTE[palette_name]
        elif palette_name == "Custom":
            colors = self._conf['color_palette']
        else:
            colors = PREDEF_PALETTE['Tango']
        fg = self.clbtnFg.get_color()
        bg = self.clbtnBg.get_color()
        palette = []
        for i in range(0, 16):
            palette.append(gtk.gdk.Color(colors[i]))
            self.clbtnPalette[i].set_color(palette[i])
        self.demoTerm.set_colors(fg, bg, palette)

    def _remove_path_from_list(self, gtktree, gtklist, conflist):
        """ Remove the path from the list

        Arguments:
          * gtktree -- the gtkTreeView
          * gtklist -- the gtkListStore where we will remove the path
          * confilst -- the "Conf" path list
        """
        model, iter_ = gtktree.get_selection().get_selected()
        conflist.remove(gtklist.get_value(iter_, 0))
        gtklist.remove(iter_)

    def _add_path_to_list(self, gtklist, path, conflist):
        """ Adds the path in the list

        Arguments:
          * path -- the path to add
          * gtklist -- the gtkListStore where we will add the path
          * confilst -- the "Conf" path list
        """
        if not path in conflist:
            gtklist.append([path])
            conflist.append(path)


class NautilusTerminal(nautilus.LocationWidgetProvider):
    """An integrated terminal for Nautilus."""

    def __init__(self):
        """The constructor."""
        self._term_height = CONF['general_defheight']
        self._term_hidden = CONF['general_starthidden']

    def get_widget(self, uri, window):
        """Returns the widgets to display to Nautilus."""
        #If it's not a local folder, don't show the teminal
        if uri[:7] != "file://":
            return
        #Transform uri to path
        path = urllib.url2pathname(uri[7:])
        #If folder not in white list
        if not CONF['folders_showinall'] and \
        not match_path(path, CONF['folders_list']):
            return
        #Change current dir
        os.chdir(path)
        #GUI
        gui = gtk.Builder()
        gui.set_translation_domain(__appname__)
        gui.add_from_file(TERMINAL_GUI_FILE)
        vboxMain = gui.get_object("vboxMain")
        #### hboxHidden ####
        hboxHidden = gui.get_object("hboxHidden")
        if self._term_hidden:
            hboxHidden.show()
        else:
            hboxHidden.hide()
        btnPref2 = gui.get_object("btnPref2")
        btnHide = gui.get_object("btnHide")
        btnShow = gui.get_object("btnShow")
        #### hboxTerm ####
        hboxTerm = gui.get_object("hboxTerm")
        if self._term_hidden:
            hboxTerm.hide()
        else:
            hboxTerm.show()
        btnInc = gui.get_object("btnInc")
        btnDec = gui.get_object("btnDec")
        btnPref = gui.get_object("btnPref")
        #terminal
        terminal = vte.Terminal()
        terminal.set_size(40, self._term_height)
        self._set_terminal(terminal)
        terminal.fork_command(CONF['general_command'])
        terminal.show()
        vpTerm = gui.get_object("vpTerm")
        vpTerm.add(terminal)
        #
        if self._term_height < 7:
            btnPref.hide()
        if self._term_height < 5:
            btnDec.hide()
        #### Signals ####
        #Button increase
        btnInc.connect("clicked", self.terminal_size_inc, terminal, btnInc, btnDec, btnPref)
        #Button decrease
        btnDec.connect("clicked", self.terminal_size_dec, terminal, btnInc, btnDec, btnPref)
        #Button preferences
        btnPref.connect("clicked", self.preferences, terminal)
        #Button preferences 2
        btnPref2.connect("clicked", self.preferences, terminal)
        #Button Hide
        btnHide.connect("clicked", self.hide, hboxHidden, hboxTerm)
        #Button Show
        btnShow.connect("clicked", self.show, hboxHidden, hboxTerm, window, terminal)
        #Terminal
        terminal.connect("child-exited", self.on_terminal_child_exited, terminal)
        #### Accel ####
        accel_group = gtk.AccelGroup()
        window.add_accel_group(accel_group)
        terminal.add_accelerator("paste-clipboard", accel_group, ord('V'), gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK, gtk.ACCEL_VISIBLE)
        terminal.add_accelerator("copy-clipboard", accel_group, ord('C'), gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK, gtk.ACCEL_VISIBLE)
        btnShow.add_accelerator("clicked", accel_group, ord('T'), gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK, gtk.ACCEL_VISIBLE)
        btnHide.add_accelerator("clicked", accel_group, ord('H'), gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK, gtk.ACCEL_VISIBLE)
        #### Return the widgets ####
        vboxMain.unparent()
        return vboxMain

    def terminal_size_inc(self, widget, terminal, btninc, btndec, btnpref):
        """Increase the terminal size.

        Arguments:
            * widget -- the widget that call this function
            * terminal -- the vte to resize
        """
        rows = terminal.get_row_count()
        if rows >= 6:
            btnpref.show()
        if rows >= 4:
            btndec.show()
        if rows == 19:
            btninc.set_sensitive(False)
        if rows < 20:
            terminal.set_size(10, rows + 1)
            self._term_height = rows + 1

    def terminal_size_dec(self, widget, terminal, btninc, btndec, btnpref):
        """Decrease the terminal size.

        Arguments:
            * widget -- the widget that call this function
            * terminal -- the vte to resize
        """
        rows = terminal.get_row_count()
        if rows <= 7:
            btnpref.hide()
        if rows <= 5:
            btndec.hide()
        btninc.set_sensitive(True)
        if rows > 4:
            terminal.set_size(10, rows - 1)
            self._term_height = rows - 1

    def preferences(self, widget, terminal):
        """Display the preferences dialog.

        Arguments:
            * widget -- the widget that call this function
            * terminal -- the vte (for applying the changes)
        """
        NautilusTerminalPref(self._set_terminal, terminal)

    def show(self, widget, hboxhidden, hboxterm, window, terminal):
        hboxhidden.hide()
        hboxterm.show()
        window.set_focus(terminal)
        self._term_hidden = False

    def hide(self, widget, hboxhidden, hboxterm):
        hboxhidden.show()
        hboxterm.hide()
        self._term_hidden = True

    def on_terminal_child_exited(self, widget, terminal):
        terminal.fork_command(CONF['general_command'])

    def _set_terminal(self, terminal):
        """Set terminal font, colors, palette,...
        Arguments:
            * terminal -- the vte
        """
        #General
        terminal.set_cursor_shape(CONF['general_cursor'])
        #Colors
        if CONF['color_palettename'] in PREDEF_PALETTE:
            colors = PREDEF_PALETTE[CONF['color_palettename']]
        elif CONF['color_palettename'] == "Custom":
            colors = CONF['color_palette']
        else:
            colors = PREDEF_PALETTE['Tango']
        fg = gtk.gdk.Color(CONF['color_text'])
        bg = gtk.gdk.Color(CONF['color_background'])
        palette = []
        for color in colors:
            palette.append(gtk.gdk.Color(color))
        terminal.set_colors(fg, bg, palette)
        #Font
        font = font = pango.FontDescription(CONF['font_name'])
        terminal.set_font(font)
        terminal.set_allow_bold(CONF['font_allowbold'])


def match_path(path, path_list):
    """Test if a folder is a sub-folder of another one in the list.

    Arguments
      * path -- path to check
      * path_list -- list of path
    """
    match = False
    #We add a slash at the end.
    if path[-1:] != "/":
        path += "/"
    for entry in path_list:
        #We add a slash at the end.
        if entry[-1:] != "/":
            entry += "/"
        if re.match(r"^" + entry + ".*", path):
            match = True
            break
    return match


CONF = Conf(CONF_FILE)


